@@ -35,6 +35,12 @@
   #:use-module (nyacc version)
   )
 
+
+;; token values for default reduction and erro, sync with parser.scm
+;; used in hashify-machine, compact-machine
+(define $default 1)
+(define $error 2)
+
 ;; @deffn {Procedure} proxy-? sym rhs
 ;; @example
 ;; (LHS (($? RHS))
@@ -256,10 +262,10 @@
 ;; @end deffn
 (define (process-spec tree)
 
-  ;; Make a new symbol. This is a helper for proxies and mid-rule-actions.
+  ;; Generate a new symbol. This is a helper for proxies and mid-rule-actions.
   ;; The counter here is the only @code{set!} in @code{process-spec}.
   ;; Otherwise, I believe @code{process-spec} is referentially transparent.
-  (define maksy
+  (define gensy
     (let ((cntr 1))
       (lambda ()
 	(let ((c cntr))
@@ -305,13 +311,13 @@
 	 `(prec . ,(reverse pll))
 	 `(assc (left ,@la) (right ,@ra) (nonassoc ,@na)))))))
 
-  ;;.@deffn make-mra-proxy sy pel act => ???
+  ;;.@deffn {Procedure} make-mra-proxy sy pel act => ???
   ;; Generate a mid-rule-action proxy.
   ;; @end deffn
   (define (make-mra-proxy sy pel act)
     (list sy (list (cons* 'action (length pel) (cdr act)))))
 
-  ;; @deffn gram-check-2 tl nl err-l
+  ;; @deffn {Procedure} gram-check-2 tl nl err-l
   ;; Check for fatal: symbol used as terminal and non-terminal.
   ;; @end deffn
   (define (gram-check-2 tl nl err-l)
@@ -331,7 +337,7 @@
 	   l))
      err-l nl))
 
-  ;; @deffn gram-check-4 ll nl err-l
+  ;; @deffn {Procedure} gram-check-4 ll nl err-l
   ;; Check for warning: unused LHS.
   ;; TODO: which don't appear in OTHER RHS, e.g., (foo (foo))
   ;; @end deffn
@@ -349,6 +355,8 @@
 
   ;; TODO: check for repeated tokens in precedence spec's: prec<, prec>
 
+  ;; IN PROGRESS: add zero-rule $accept : start-symbol $end
+
   (let* ((gram (assq-ref tree 'grammar))
 	 (start-symbol (and=> (assq-ref tree 'start) atomize))
 	 (start-rule (lambda () (list start-symbol)))
@@ -368,9 +376,9 @@
 	       (@l (list		; attributes per prod' rule
 		    `((rhs . ,(vector start-symbol))
 		      (ref . all) (act 1 $1))))
-	       (tl (append '($code-comm $lone-comm $error $end) ; terminals
-			   (or (assq-ref tree 'reserve) '())))
-	       (nl (list start-symbol))	; set of non-terminals
+	       (tl (cons* '$error '$end ; terminals
+			  (or (assq-ref tree 'reserve) '())))
+	       (nl (list start-symbol)) ; set of non-terminals
 	       ;;
 	       (head gram)	       ; head of unprocessed productions
 	       (prox '())	       ; proxy productions for MRA
@@ -393,7 +401,7 @@
 	  ((action)
 	   (if (pair? (cdr rhs))
 	       ;; mid-rule action: generate a proxy (car act is # args)
-	       (let* ((sy (maksy))
+	       (let* ((sy (gensy))
 		      (pr (make-mra-proxy sy pel (cdar rhs))))
 		 (iter ll @l tl (cons sy nl) head (cons pr prox)
 		       lhs tail rhs-l attr (cons sy pel) (cdr rhs)))
@@ -401,7 +409,7 @@
 	       (iter ll @l tl nl head prox lhs tail
 		     rhs-l (acons 'action (cdar rhs) attr) pel (cdr rhs))))
 	  ((proxy)
-	   (let* ((sy (maksy))
+	   (let* ((sy (gensy))
 		  (pf (cadar rhs))	; proxy function
 		  (p1 (pf sy (cddar rhs))))
 	     (iter ll @l tl (cons sy nl) head (cons p1 prox) lhs
@@ -1510,13 +1518,15 @@
 ;; Others are provided as strings.
 ;; @end enumerate
 ;; The procedure @code{hashify-machine} will convert the cdrs to integers.
-;; Test: "$abc" => ("$abc" '$abc) '$abc => ('$abc . '$abc)
+;; Test: "$abc" => ("$abc" '$abc) '$abc => ('$abc . '$abc) @*
+;; Note: Adding in $start sym for interactive parser helper.
 ;; @end deffn
 (define (gen-match-table mach)
-  (cons
-   (cons 'mtab (map (lambda (term) (cons term (atomize term)))
+  (acons 'mtab
+	 (cons (cons '$start (lalr-start mach))
+	       (map (lambda (term) (cons term (atomize term)))
 		    (assq-ref mach 'terminals)))
-   mach))
+	 mach))
 
 
 ;; @deffn {Procedure} add-recovery-logic! mach => mach
@@ -1636,7 +1646,7 @@
       (vector-for-each
        (lambda (ix pat)
 	 (for-each
-	  (lambda (a) (if (or (eqv? (car a) '$default) (eqv? (car a) -1))
+	  (lambda (a) (if (or (eqv? (car a) '$default) (eqv? (car a) 1))
 			  (abort-to-prompt 'got-it)))
 	  pat))
        (assq-ref mach 'pat-v))
@@ -1644,23 +1654,18 @@
     ;; otherwise, return #t.
     (lambda () #t)))
 
-;; The list of tokens that do not get absorbed into default reductions.
-;; See @code{compact-machine} below.
-(define default-keepers '($error $lone-comm $code-comm $end))
-
-;; @deffn {Procedure} compact-machine mach [#:keep 3] [#:keepers '()] => mach
+;; @deffn {Procedure} compact-machine mach [#:keep 0] [#:keepers '()] => mach
 ;; A "filter" to compact the parse table.  For each state this will replace
 ;; the most populus set of reductions of the same production rule with a
-;; default production.  However, reductions triggered by user-specified keepers
-;; and the default keepers -- @code{'$error}, @code{'$end}, @code{'$lone-comm}
-;; and @code{'$lone-comm} are not counted.  The parser will want to treat
-;; errors and comments separately so that they can be trapped (e.g.,
-;; unaccounted comments are skipped).
+;; default production.  However, reductions triggered by @var{keepers} and
+;; the required keeper -- @code{'$error} -- are not counted.  The keepers
+;; can then be trapped by the parser (e.g., to skip un-accounted comments).
 ;; @end deffn
-(define* (compact-machine mach #:key (keep 3) (keepers '()))
+(define* (compact-machine mach #:key (keep 0) (keepers '()))
+  (if (< keep 0) (error "expecting keep > 0"))
   (let* ((pat-v (assq-ref mach 'pat-v))
 	 (nst (vector-length pat-v))
-	 (hashed (number? (caar (vector-ref pat-v 0)))) ; been hashified?
+	 (hashed (number? (caar (vector-ref pat-v 0)))) ; hashified?
 	 (reduce? (if hashed
 		      (lambda (a) (and (number? a) (negative? a)))
 		      (lambda (a) (eq? 'reduce (car a)))))
@@ -1670,11 +1675,11 @@
 			 (lambda (a r) (and (eq? 'reduce (car a))
 					    (eqv? r (cdr a))))))
 	 (mk-default (if hashed
-			 (lambda (r) (cons -1 (- r)))
+			 (lambda (r) (cons $default (- r)))
 			 (lambda (r) `($default reduce . ,r))))
 	 (mtab (assq-ref mach 'mtab))
 	 (keepers (map (lambda (k) (assq-ref mtab k))
-		       (append keepers default-keepers))))
+		       (cons '$error keepers))))
 
     ;; Keep an a-list mapping reduction prod-rule => count.
     (let iter ((sx nst) (trn-l #f) (cnt-al '()) (p-max '(0 . 0)))
@@ -1758,7 +1763,8 @@
 ;; list through the ident-reader.
 ;; NOTE: The parser is hardcoded to assume that the phony token for the
 ;; default (reduce) action is @code{'$default} for unhashed machine or
-;; @code{-1} for a hashed machine.
+;; @code{-1} for a hashed machine.  In addition, we use @code{-2} for
+;; @code{$end}.
 
 ;; NEW: need to add reduction of ERROR
 
@@ -1777,9 +1783,12 @@
 	     (non-terms (assq-ref mach 'non-terms))
 	     (lhs-v (assq-ref mach 'lhs-v))
 	     (sm ;; = (cons sym->int int->sym)
-	      (let iter ((si (list (cons '$default -1)))
-			 (is (list (cons -1 '$default)))
-			 (ix 1) (tl terminals) (nl non-terms))
+	      (let iter ((si (list (cons '$default $default)
+				   (cons '$error $error)))
+			 (is (list (cons $default '$default)
+				   (cons '$error $error)))
+			 (ix (1+ (max $default $error 0)))
+			 (tl terminals) (nl non-terms))
 		(if (null? nl) (cons (reverse si) (reverse is))
 		    (let* ((s (atomize (if (pair? tl) (car tl) (car nl))))
 			   (tl1 (if (pair? tl) (cdr tl) tl))
@@ -1958,14 +1967,14 @@
 				((positive? p) 'shift)
 				((negative? p) 'reduce)
 				(else 'accept)))
-		      (gt (and=> p abs)))
+		      (gt (if p (abs p) "")))
 		 (fmt port "\t\t~A => ~A ~A\n"
 		      (elt->str (assq-ref i->s sy) terms)
 		      pa gt))))
 	 pat)
 	(for-each			; action, print it
 	 (lambda (ra)
-	   ;; FIX: should indicate if precedence removed user rule or default
+	   ;; FIX: indicate if precedence removed by user rule or default
 	   (fmt port "\t\t[~A => ~A ~A] REMOVED by ~A\n"
 		(elt->str (caar ra) terms) (cadar ra) (cddar ra)
 		(case (cdr ra)
